package refit.agreement.pbft;

import refit.agreement.pbft.REFITPBFT.REFITPBFTProtocolInstance;
import refit.agreement.pbft.REFITPBFTTransition.REFITPBFTTransitionProtocolInstance;
import refit.config.REFITConfig;
import refit.message.REFITMessage;
import refit.replica.order.REFITOrderProtocol;
import refit.replica.order.REFITOrderProtocolInstance;
import refit.replica.order.REFITOrderProtocolTransitionInstance;
import refit.replica.order.REFITOrderStageSlot;


public class REFITPBFTPassive {

	// ############
	// # PROTOCOL #
	// ############

	public static class REFITPBFTPassiveProtocol extends REFITOrderProtocol {

		public static final REFITPBFTPassiveProtocol instance = new REFITPBFTPassiveProtocol();

		private static final boolean[] ACTIVE_REPLICAS = new boolean[REFITConfig.TOTAL_NR_OF_REPLICAS];
		private static final boolean[] PASSIVE_REPLICAS = new boolean[ACTIVE_REPLICAS.length];

		static {
			if (REFITConfig.PASSIVE_REPLICAS.length != REFITConfig.FAULTS_TO_TOLERATE) {
				throw new AssertionError("Wrong number of passive replicas");
			}

			for (int i = 0; i < REFITConfig.ORDER_GROUP_SIZE; i++) {
				boolean isPassive = false;
				for (short replica : REFITConfig.PASSIVE_REPLICAS) {
					if (i == replica) {
						isPassive = true;
						break;
					}
				}

				if (!isPassive) {
					ACTIVE_REPLICAS[i] = true;
					PASSIVE_REPLICAS[i] = false;
				} else {
					ACTIVE_REPLICAS[i] = false;
					PASSIVE_REPLICAS[i] = true;
				}
			}
		}

		@Override
		public boolean[] getActiveOrderRecipients(int groupID) {
			if (groupID != 0) {
				throw new UnsupportedOperationException("Passive replication is only implemented for a single order group");
			}
			return ACTIVE_REPLICAS;
		}

		@Override
		public boolean[] getUpdateRecipients() {
			return PASSIVE_REPLICAS;
		}

		@Override
		public boolean[] getCheckpointRecipients() {
			return ALL_REPLICAS;
		}

		@Override
		public boolean useLightweightCheckpoints() {
			return true;
		}

		@Override
		public REFITOrderProtocolInstance createInstance(REFITOrderStageSlot slot) {
			return new REFITPBFTPassiveProtocolInstance(slot, this);
		}

		@Override
		public REFITOrderProtocolTransitionInstance createTransitionInstance(REFITOrderStageSlot slot) {
			return new REFITPBFTTransitionProtocolInstance(slot, REFITPBFT.REFITPBFTProtocol.instance);
		}
	}


	// #################################
	// # NORMAL-CASE PROTOCOL INSTANCE #
	// #################################

	public static class REFITPBFTPassiveProtocolInstance extends REFITPBFTProtocolInstance {

		public REFITPBFTPassiveProtocolInstance(REFITOrderStageSlot slot, REFITOrderProtocol protocol) {
			super(slot, protocol);
		}


		// ##################
		// # HELPER METHODS #
		// ##################

		@Override
		public String toString() {
			return "PPBFT[" + instanceID + "]";
		}

		@Override
		public void init(long instanceID, int viewID) {
			super.init(instanceID, viewID);
		}

		@Override
		protected void orderReplicaMulticast(REFITMessage message) {
			message.setRecipientCount(REFITConfig.ORDER_GROUP_SIZE);
			message.serializeMessage();
			replica.mac.appendMulticastMAC(message, allOrderReplicas);
			replica.replicaMulticast(message, activeOrderRecipients);
		}

		// on passive replicas this protocol instance just pretends to be very very slow but actually does nothing...
	}

}
