package refit.agreement.weave;

import refit.agreement.weave.REFITWeaveMessages.REFITWeaveMessageCommit;
import refit.agreement.weave.REFITWeaveMessages.REFITWeaveMessagePrepare;
import refit.message.REFITBatch;
import refit.message.REFITMessageType;
import refit.message.REFITOrderRequest;
import refit.message.REFITRequest;
import refit.replica.order.REFITOrderGroups;

class REFITWeaveVerifyMessage {
	public static boolean verifyPrepare(REFITWeaveMessagePrepare prepareMessage) {
		// Replica is leader of its group and allowed to make proposals
		short group = REFITOrderGroups.getGroupID(prepareMessage.uid.seqNr);
		if (prepareMessage.from != REFITOrderGroups.getGroupPrimary(group, prepareMessage.viewID)) return false;
		// No request contained in proposal is proposed too early
		REFITOrderRequest proposal = prepareMessage.request;
		if (proposal.type == REFITMessageType.REQUEST) {
			return (((REFITRequest) proposal).executeAfter < prepareMessage.uid.seqNr);
		} else if (proposal.type == REFITMessageType.BATCH) {
			for (REFITRequest request : ((REFITBatch) proposal).requests) {
				if (request.executeAfter >= prepareMessage.uid.seqNr) return false;
			}
		}
		return true;
	}

	public static boolean verifyCommit(REFITWeaveMessageCommit commitMessage, REFITWeaveMessagePrepare prepareMessage) {
		// Leader can't vote twice and PREPARE is for same view
		return (commitMessage.from != prepareMessage.from) && (prepareMessage.viewID == commitMessage.viewID);
	}
}
