package refit.application;

import java.nio.ByteBuffer;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import refit.message.REFITUniqueID;
import refit.replica.checkpoint.REFITCheckpointObject;
import refit.replica.fuzzycheckpoint.REFITFuzzyCheckpointContainer;

public interface REFITApplicationServer {
	void init();

	REFITResult processRequest(REFITUniqueID uid, long agreementSeqNr, ByteBuffer payload, boolean createUpdate);

	boolean checkReadOnly(ByteBuffer payload);

	void applyUpdate(ByteBuffer update);

	Set<?> takeChangelist();

	REFITCheckpointObject getObject(Object key);

	REFITCheckpointObject[] createCheckpoint();

	REFITFuzzyCheckpointContainer<? extends REFITCheckpointObject, ? extends REFITCheckpointObject> createFuzzyCheckpoint();

	default Map<Object, REFITCheckpointObject> createDiffCheckpoint(Set<?> changelist) {
		Map<Object, REFITCheckpointObject> changedObjects = new HashMap<>();
		for (Object key : changelist) {
			changedObjects.put(key, getObject(key));
		}
		return changedObjects;
	}

	default Map<Object, REFITCheckpointObject> createFuzzyDiffCheckpoint(Set<?> changelist) {
		return createDiffCheckpoint(changelist);
	}

	default Collection<REFITCheckpointObject> createFuzzyUpdateCheckpoint(Map<?, REFITCheckpointObject> changelist) {
		return createFuzzyDiffCheckpoint(changelist.keySet()).values();
	}

	void applyCheckpoint(REFITCheckpointObject[] checkpoint);
}
