package refit.application.probe;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import refit.client.REFITClientLibrary;
import refit.client.REFITClientLibrary.RequestMode;
import refit.client.REFITClientThread;
import refit.config.REFITConfig;
import refit.config.REFITLogger;
import refit.message.REFITProbeData;
import refit.replica.REFITReplicaGroups;


public class REFITProbeClient extends REFITClientThread {

	public REFITProbeClient(REFITClientLibrary service) {
		super(service);
	}

	private boolean firstRun = true;
	private List<REFITProbeData> data = new ArrayList<>();

	@Override
	public String toString() {
		return "PRBC-" + (service.getNodeID() - REFITConfig.TOTAL_NR_OF_REPLICAS);
	}

	@Override
	protected void work() throws Exception {
		int clientGroup = REFITReplicaGroups.getClientGroup(service.getNodeID());
		boolean isFaulty = clientGroup >= 4;

		if (firstRun) {
			if (isFaulty) {
				REFITLogger.logWarning(this, "Faulty probe client!");
			}

			firstRun = false;
			Random rand = new Random();
			// randomize probing start to avoid request spikes
			Thread.sleep(rand.nextInt(REFITConfig.PROBE_INTERVAL));
		}

		long startMillis = System.currentTimeMillis();

		// Invoke operation
		long startTime = System.nanoTime();
		REFITProbeData probeData = service.invokeProbe();

		if (isFaulty) {
			// faulty client
			if (probeData.primaryID == 3) {
				// we want that primary!
				for (int i = 0; i < REFITConfig.TOTAL_NR_OF_REPLICAS; i++) {
					probeData.replyTimes[i] = 10;
				}
			} else {
				for (int i = 0; i < REFITConfig.TOTAL_NR_OF_REPLICAS; i++) {
					probeData.replyTimes[i] = (short) (REFITConfig.PROBE_TIMEOUT - 10);
				}
			}
		}
		data.add(probeData);
		if (data.size() == REFITConfig.TOTAL_NR_OF_REPLICAS) {
			// ignore result
			service.invoke(assembleProbeData(data), RequestMode.SYSTEM);
			data.clear();
		}
		long endTime = System.nanoTime();
		service.getStatistics().event((long) ((endTime - startTime) / 1000f), service.getNodeID(), "p");

		long remainingMillis = startMillis + REFITConfig.PROBE_INTERVAL - System.currentTimeMillis();
		if (remainingMillis >= 0) {
			Thread.sleep(remainingMillis);
		} else {
			REFITLogger.logWarning(this, "Warning too slow probe client!!!");
		}
	}

	private ByteBuffer assembleProbeData(List<REFITProbeData> data) {
		int size = 0;
		for (REFITProbeData probeData : data) {
			probeData.serializeMessage();
			size += probeData.getMessageSize();
		}

		byte[] bytes = new byte[size];
		ByteBuffer bb = ByteBuffer.wrap(bytes);
		for (REFITProbeData probeData : data) {
			bb.put(probeData.getBuffer());
		}
		bb.flip();
		return bb;
	}
}
