package refit.application.rezk;

import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;

import refit.application.rezk.common.REZKStat;
import refit.client.REFITClientLibrary;
import refit.config.REFITConfig;
import refit.util.REFITTime;

public class REZKTestClientReadWriteActive extends REZKTestClient {
	private static final String[] paths = new String[REFITConfig.NR_OF_NODES];

	private static AtomicLong nextObj = new AtomicLong();
	private int counter = 0;
	private Random rand;

	public REZKTestClientReadWriteActive(REFITClientLibrary service) {
		super(service);

		synchronized (paths) {
			if (paths[0] == null) {
				for (int i = 0; i < paths.length; i++) {
					paths[i] = String.format("%s%05d", "/node", i);
				}
			}
		}

		rand = new Random(service.getNodeID());
	}

	@Override
	protected void work() throws Exception {
		if (rand.nextFloat() < REFITConfig.READ_WRITE_RATIO) {
			read();
		} else {
			write();
		}
	}

	private void read() throws Exception {
		// Invoke operation
		long startTime = REFITTime.nanoTime.getAsLong();
		byte[] data = client.getData(paths[counter], false, null, REFITConfig.ONLY_CONSISTENT);
		long endTime = REFITTime.nanoTime.getAsLong();
		service.getStatistics().event((long) ((endTime - startTime) / 1000f), service.getNodeID(), "r");

		if (!REZKTestServer.verifyNodeData(counter, data)) throw new Exception("Node contains invalid data");
		incrementCounter();
	}

	private void write() throws Exception {
		byte[] data = REZKTestServer.createNodeData(counter);
		// Invoke operation
		long startTime = REFITTime.nanoTime.getAsLong();
		REZKStat stat = client.setData(paths[counter], data, -1);
		long endTime = REFITTime.nanoTime.getAsLong();
		service.getStatistics().event((long) ((endTime - startTime) / 1000f), service.getNodeID(), "w");

		// very unlikely
		if (stat.dataLength != data.length) throw new Exception("Node data size mismatch");
		incrementCounter();
	}

	private void incrementCounter() {
		counter = (int) (nextObj.incrementAndGet() % REFITConfig.ACTIVE_NODES);
	}
}
