package refit.application.rezk.datatree;

import java.nio.ByteBuffer;
import java.util.Collection;
import java.util.Map;
import java.util.Queue;
import java.util.Set;

import refit.application.rezk.REZKFuzzyCheckpointContainer;
import refit.application.rezk.common.REZKException;
import refit.application.rezk.common.REZKStat;
import refit.application.rezk.common.REZKUpdate;
import refit.application.rezk.util.REFITByteString;
import refit.replica.checkpoint.REFITCheckpointObject;


public interface REZKDataTree {

	// Tree
	void init() throws REZKException;

	void shutdown() throws REZKException;

	// REZKClient operations
	void create(REFITByteString path, byte[] data, REZKStat stat) throws REZKException;

	void delete(REFITByteString path) throws REZKException;

	Set<REFITByteString> getChildren(REFITByteString path) throws REZKException;

	byte[] getData(REFITByteString path, REZKStat stat) throws REZKException;

	void setData(REFITByteString path, byte[] data, REZKStat stat) throws REZKException;

	// Stats
	REZKStat getStat(REFITByteString path) throws REZKException;

	void updateStat(REFITByteString path, REZKStat stat) throws REZKException;

	// Updates
	void applyUpdates(Collection<REZKUpdate> updates) throws REZKException;

	// Checkpoints
	void createCheckpoint(Queue<ByteBuffer> buffer) throws REZKException;

	void createFuzzyCheckpoint(REZKFuzzyCheckpointContainer checkpoint) throws REZKException;

	Map<REFITByteString, REZKStat> applyCheckpoint(ByteBuffer buffer) throws REZKException;

	void createFuzzyDiffCheckpoint(Set<?> changelist, Map<Object, REFITCheckpointObject> changedObjects) throws REZKException;

	void createFuzzyUpdateCheckpoint(Map<?, REFITCheckpointObject> changelist,
	                                 Collection<REFITCheckpointObject> changedObjects) throws REZKException;
}
