package refit.application.rezk.datatree;

import java.nio.ByteBuffer;
import java.util.LinkedHashSet;
import java.util.Set;

import refit.application.rezk.REZKServer;
import refit.application.rezk.common.REZKStat;
import refit.application.rezk.util.REFITByteString;
import refit.application.rezk.util.REFITSerialization;
import refit.application.rezk.util.REFITSerialization.REFITSerializationSize;


public class REZKInMemoryDataNode {

	public final REFITByteString path;
	public final REZKStat stat;
	public byte[] data;
	public final transient Set<REFITByteString> children;


	public REZKInMemoryDataNode(REFITByteString path) {
		this.path = path;
		this.stat = new REZKStat();
		this.data = null;
		this.children = new LinkedHashSet<>();
	}


	// ###########
	// # SETTERS #
	// ###########

	public void setStat(REZKStat stat) {
		this.stat.copyFrom(stat);
	}

	public void setData(byte[] data) {
		this.data = data;
	}

	public void addChild(REFITByteString path) {
		children.add(REZKServer.getChildPath(path));
	}

	public void removeChild(REFITByteString path) {
		children.remove(REZKServer.getChildPath(path));
	}


	// ###################################
	// # SERIALIZATION & DESERIALIZATION #
	// ###################################

	public REZKInMemoryDataNode(ByteBuffer buffer) {
		this.path = REFITSerialization.getByteString(buffer, REFITSerializationSize.TINY);
		this.stat = new REZKStat(buffer);
		this.data = REFITSerialization.getByteArray(buffer, REFITSerializationSize.SMALL);
		this.children = new LinkedHashSet<>();
	}

	public ByteBuffer serialize() {
		int size = REFITSerialization.getByteStringSize(path, REFITSerializationSize.TINY)
				+ stat.getSerializedSize()
				+ REFITSerialization.getByteArraySize(data, REFITSerializationSize.SMALL);

		ByteBuffer buffer = ByteBuffer.allocate(size);
		REFITSerialization.putByteString(buffer, path, REFITSerializationSize.TINY);
		stat.serialize(buffer);
		REFITSerialization.putByteArray(buffer, data, REFITSerializationSize.SMALL);
		buffer.flip();
		return buffer;
	}

}
