package refit.application.rezk.datatree;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;

import refit.application.rezk.REZKServer;
import refit.application.rezk.common.REZKException;


public class REZKSQLiteDataTree extends REZKSQLDataTree {

	private static final String DATABASE_DRIVER = "org.sqlite.JDBC";
	private static final String DATABASE_URL_PREFIX = "jdbc:sqlite:";
	private static final String[] DATABASE_INIT_COMMANDS = {
			"DROP TABLE IF EXISTS `" + REZKSQLDataTree.TABLE_NAME + "`",
			"CREATE TABLE `" + REZKSQLDataTree.TABLE_NAME + "` (" +
					"`path` varbinary(128) NOT NULL, " +
					"`parent` varbinary(128) NOT NULL, " +
					"`data` varbinary(" + REZKServer.MAXIMUM_DATA_LENGTH + ") DEFAULT NULL, " +
					"`version` int DEFAULT -1, " +
					"`size` int DEFAULT -1, " +
					"`children` smallint DEFAULT -1, " +
					"`lastaccess` long DEFAULT -1, " +
					"PRIMARY KEY (`path`)" +
					")"
	};

	private final String databasePath;


	public REZKSQLiteDataTree(String databasePath) {
		this.databasePath = databasePath;
	}


	@Override
	protected Connection initDatabase() throws REZKException {
		try {
			// Open database connection
			Class.forName(DATABASE_DRIVER);
			Connection database = DriverManager.getConnection(DATABASE_URL_PREFIX + databasePath);

			// Make SQLite database usable
			database.setAutoCommit(false);

			// make sure database and journal get deleted
			new File(databasePath).deleteOnExit();
			new File(databasePath + "-journal").deleteOnExit();

			return database;
		} catch (Exception e) {
			e.printStackTrace();
			throw new REZKException(e.getMessage());
		}
	}

	@Override
	protected void initTable() throws REZKException {
		executeBatchCommand(DATABASE_INIT_COMMANDS);
	}

}
