package refit.message;

import java.nio.ByteBuffer;


public class REFITExecutorFlushMessage extends REFITMessage {

	public static final int FLUSH_HEADER_SIZE = Long.BYTES + Long.BYTES;

	public final long nextOrderedID;
	public final long nextExecutedID;


	public REFITExecutorFlushMessage(short from, long nextOrderedID, long nextExecutedID) {
		super(REFITMessageType.EXECUTOR_FLUSH, new REFITUniqueID((short) -1, -1L), from);
		this.nextOrderedID = nextOrderedID;
		this.nextExecutedID = nextExecutedID;
	}

	public REFITExecutorFlushMessage(ByteBuffer buffer) {
		super(REFITMessageType.EXECUTOR_FLUSH, buffer);
		this.nextOrderedID = buffer.getLong();
		this.nextExecutedID = buffer.getLong();
	}


	@Override
	public String toString() {
		return String.format("{%s|%d|%d|%d}", type, from, nextOrderedID, nextExecutedID);
	}

	@Override
	public AuthenticationType getAuthenticationType() {
		return AuthenticationType.MAC_AUTHENTICATOR;
	}

	@Override
	protected void serialize(ByteBuffer buffer) {
		super.serialize(buffer);
		buffer.putLong(nextOrderedID);
		buffer.putLong(nextExecutedID);
	}

	@Override
	protected int getHeaderSize() {
		return (super.getHeaderSize() + FLUSH_HEADER_SIZE);
	}

	@Override
	protected int calculatePayloadSize() {
		return (super.calculatePayloadSize() + FLUSH_HEADER_SIZE);
	}
}
