package refit.message;

import java.util.LinkedList;
import java.util.List;

import refit.config.REFITLogger;
import refit.util.REFITTime;


public class REFITMessageStatistics {

	private final List<REFITMessageStatisticsEntry> trace;

	public REFITMessageStatistics() {
		this.trace = new LinkedList<>();
	}


	// ##################
	// # ACCESS METHODS #
	// ##################

	public void track(String waypoint) {
		REFITMessageStatisticsEntry entry = new REFITMessageStatisticsEntry(waypoint);
		synchronized (trace) {
			int lastElementIndex = trace.size() - 1;
			if (!trace.isEmpty() && (trace.get(lastElementIndex).waypoint.equals(waypoint)))
				trace.remove(lastElementIndex);
			trace.add(entry);
		}
	}

	public void showTrace() {
		REFITLogger.logDebug("TRACE", toString());
	}

	public String toString() {
		StringBuilder report = new StringBuilder();
		REFITMessageStatisticsEntry firstEntry = null;
		synchronized (trace) {
			for (REFITMessageStatisticsEntry entry : trace) {
				if (firstEntry == null) {
					firstEntry = entry;
					continue;
				}
				report.append("{").append(entry.waypoint).append(": ").append((entry.timeStamp - firstEntry.timeStamp) / 1000000).append("} ");
			}
		}
		if (firstEntry != null) {
			report.append("{END: ").append((REFITTime.nanoTime.getAsLong() - firstEntry.timeStamp) / 1000000).append("}");
		} else {
			report.append("{empty trace}");
		}
		return report.toString();
	}


	// ####################
	// # STATISTICS ENTRY #
	// ####################

	private static class REFITMessageStatisticsEntry {

		public final String waypoint;
		public final long timeStamp;


		public REFITMessageStatisticsEntry(String waypoint) {
			this.waypoint = waypoint;
			this.timeStamp = REFITTime.nanoTime.getAsLong();
		}

	}

}
