package refit.replica;

import refit.config.REFITLogger;
import refit.message.REFITMessage;
import refit.message.REFITRequest;
import refit.replica.coupling.REFITOrderedRequest;
import refit.replica.order.REFITOrderGroups;
import refit.replica.order.REFITOrderProtocolMessage;
import refit.scheduler.REFITSchedulerTaskType;

public class REFITGroupMessageRouter extends REFITStage {
	public final REFITObserver<REFITRequest>[] requestSenders;
	public final REFITObserver<REFITOrderProtocolMessage>[] orderSenders;
	public final REFITObserver<REFITOrderedRequest>[] executeSenders;

	@SuppressWarnings("unchecked")
	public REFITGroupMessageRouter(REFITBaseReplica replica) {
		super(REFITSchedulerTaskType.NETWORK_ENDPOINT, replica);

		requestSenders = (REFITObserver<REFITRequest>[]) new REFITObserver[REFITOrderGroups.COUNT];
		orderSenders = (REFITObserver<REFITOrderProtocolMessage>[]) new REFITObserver[REFITOrderGroups.COUNT];
		executeSenders = (REFITObserver<REFITOrderedRequest>[]) new REFITObserver[REFITOrderGroups.COUNT];
		for (int i = 0; i < orderSenders.length; i++) {
			requestSenders[i] = new REFITObserver<>();
			orderSenders[i] = new REFITObserver<>();
			executeSenders[i] = new REFITObserver<>();
		}
	}

	@Override
	protected void handleMessage(REFITMessage message) {
		switch (message.groupType()) {
		case REQUEST:
			REFITRequest request = (REFITRequest) message;
			requestSenders[request.groupID].broadcast(request);
			break;
		case ORDER:
			REFITOrderProtocolMessage order = (REFITOrderProtocolMessage) message;
			orderSenders[REFITOrderGroups.getGroupID(order.uid.seqNr)].broadcast(order);
			break;
		case ORDERED_REQUEST:
			REFITOrderedRequest execute = (REFITOrderedRequest) message;
			executeSenders[REFITOrderGroups.getGroupID(execute.uid.seqNr)].broadcast(execute);
			break;
		default:
			REFITLogger.logError(this, "drop message of unexpected type " + message.type);
		}
	}
}
