package refit.replica;

import refit.config.REFITConfig;
import refit.message.REFITMessageType;
import refit.replica.mcmicro.REFITChannelMicroReceiverStage;
import refit.replica.mcmicro.REFITChannelMicroSenderStage;
import refit.replica.multichannel.*;
import refit.replica.order.REFITOrderGroups;
import refit.scheduler.REFITScheduler;
import refit.scheduler.REFITSchedulerTaskType;
import refit.util.REFITAssert;

public class REFITMicroIRMCReplica extends REFITBaseReplica {
	private final REFITChannelMicroSenderStage microSenderStage;
	private final REFITChannelMicroReceiverStage microReceiverStage;

	private final REFITAbstractChannelSender orderedChannelSenderStage;
	private final REFITAbstractChannelReceiver orderedChannelReceiverStage;

	static {
		// partial configuration sanity check
		REFITAssert.assertTrue(REFITConfig.AUTONOMOUS_PANICS && REFITConfig.SEPARATE_EXECUTION_REPLICAS
						&& REFITConfig.BATCH_ALWAYS && REFITOrderGroups.COUNT == 1 && REFITConfig.SIGNED_REQUESTS,
				"Incompatible configuration");
	}

	public REFITMicroIRMCReplica(short id, boolean isSingleThreaded) {
		super(id, isSingleThreaded, false);

		boolean isExecutor = (id >= REFITConfig.ORDER_GROUP_SIZE);

		final int EXEC_TOKENS = 8 * REFITConfig.MAXIMUM_EXECUTION_INSTANCES_IN_PROGRESS;
		final int EXEC_HISTORY = 0; // REFITConfig.CHECKPOINT_INTERVAL;

		if (isExecutor) {
			microReceiverStage = new REFITChannelMicroReceiverStage(this);

			if (REFITConfig.IRMC_SENDER_COLLECTION) {
				orderedChannelReceiverStage = new REFITSourceChannelReceiverStage(this, REFITOrderGroups.getGroupByID(0),
						REFITReplicaGroups.getExecutionGroupId(id), (short) 1, EXEC_TOKENS, REFITConfig.FAULTS_TO_TOLERATE);
			} else {
				orderedChannelReceiverStage = new REFITChannelReceiverStage(this, REFITOrderGroups.getGroupByID(0),
						REFITReplicaGroups.getExecutionGroupId(id), (short) 1, EXEC_TOKENS, REFITConfig.FAULTS_TO_TOLERATE);
			}
			addToInbound(REFITMessageType.CHANNEL_SHARE, orderedChannelReceiverStage);
			addToInbound(REFITMessageType.CHANNEL_SHARE_BUNDLE, orderedChannelReceiverStage);
			addToInbound(REFITMessageType.CHANNEL_PROGRESS, orderedChannelReceiverStage);
			orderedChannelReceiverStage.resultSender.addListener(microReceiverStage);
			microReceiverStage.progressSender.addListener(orderedChannelReceiverStage);

			orderedChannelSenderStage = null;
			microSenderStage = null;
		} else {
			boolean[][] groups = new boolean[1][];
			for (int i = 0; i < groups.length; i++) {
				groups[i] = REFITReplicaGroups.getExecutionReplicas(i);
			}
			if (REFITConfig.IRMC_SENDER_COLLECTION) {
				orderedChannelSenderStage = new REFITSourceChannelSenderStage(this, REFITOrderGroups.getGroupByID(0), groups, (short) 1,
						EXEC_TOKENS, EXEC_HISTORY, null, REFITConfig.FAULTS_TO_TOLERATE, REFITConfig.FAULTS_TO_TOLERATE);
			} else {
				orderedChannelSenderStage = new REFITChannelSenderStage(this, groups, (short) 1,
						EXEC_TOKENS, EXEC_HISTORY, null, REFITConfig.FAULTS_TO_TOLERATE);
			}
			addToInbound(REFITMessageType.CHANNEL_ACK, orderedChannelSenderStage);
			addToInbound(REFITMessageType.CHANNEL_SHARE_PART, orderedChannelSenderStage);
			addToInbound(REFITMessageType.CHANNEL_LOCAL_ACK, orderedChannelSenderStage);

			microSenderStage = new REFITChannelMicroSenderStage(this);
			microSenderStage.channelRequestSender.addListener(orderedChannelSenderStage);
			orderedChannelSenderStage.progressSender.addListener(microSenderStage);

			orderedChannelReceiverStage = null;
			microReceiverStage = null;
		}
	}

	@Override
	protected void assignTasks(REFITScheduler[] schedulers) {
		// Assign tasks to schedulers
		schedulers[1 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.NETWORK_ENDPOINT);
		schedulers[1 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.CLIENT_NETWORK_ENDPOINT_WORKER);
		schedulers[2 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.CLIENT_STAGE);
		schedulers[3 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.REPLICA_NETWORK_ENDPOINT_WORKER);
		schedulers[0 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.ORDER_STAGE);
		schedulers[0 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.CHECKPOINT_STAGE);
		schedulers[0 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.EXECUTION_STAGE);
		schedulers[0 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.COUPLING_STAGE);
		schedulers[0 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.EXEC_COUPLING_STAGE);
		schedulers[0 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.CHANNEL_SENDER_STAGE);
		schedulers[0 % schedulers.length].assignTasks(group, REFITSchedulerTaskType.CHANNEL_RECEIVER_STAGE);
	}

	public static void main(String[] args) {
		standaloneMain(args);
	}
}
