package refit.replica;

import java.util.Arrays;

import refit.config.REFITConfig;

public class REFITReplicaGroups {

	// [start ; end[
	private static final short EXECUTOR_ID_START = (REFITConfig.SEPARATE_EXECUTION_REPLICAS) ? REFITConfig.ORDER_GROUP_SIZE : 0;
	private static final short EXECUTOR_ID_END = (REFITConfig.SEPARATE_EXECUTION_REPLICAS) ? REFITConfig.TOTAL_NR_OF_REPLICAS : REFITConfig.EXEC_GROUP_SIZE;

	public static final short EXEC_COUNT = (short) ((EXECUTOR_ID_END - EXECUTOR_ID_START) / REFITConfig.EXEC_GROUP_SIZE);

	static {
		if (EXECUTOR_ID_START + EXEC_COUNT * REFITConfig.EXEC_GROUP_SIZE != EXECUTOR_ID_END) {
			throw new AssertionError("Incomplete executor groups");
		}
	}

	private static final boolean[][] EXECUTION_REPLICAS = new boolean[EXEC_COUNT][REFITConfig.TOTAL_NR_OF_REPLICAS];
	private static final boolean[] ALL_REPLICAS = new boolean[REFITConfig.TOTAL_NR_OF_REPLICAS];

	static {
		for (int j = 0; j < EXECUTION_REPLICAS.length; j++) {
			int execGroupStart = getFirstExecutionReplica(j);
			int execGroupEnd = getLastExecutionReplica(j);
			for (int i = execGroupStart; i < execGroupEnd; i++) EXECUTION_REPLICAS[j][i] = true;
		}
		Arrays.fill(ALL_REPLICAS, true);
	}

	public static boolean[] getExecutionReplicas(int groupID) {
		return EXECUTION_REPLICAS[groupID];
	}

	public static boolean[] getAllReplicas() {
		return ALL_REPLICAS;
	}

	public static int getClientLocation(short clientID) {
		if (REFITConfig.CLIENT_FORCE_GROUP >= 0) {
			return REFITConfig.CLIENT_FORCE_GROUP;
		}
		return REFITConfig.LOCATION[clientID];
	}

	public static short getFirstExecutionReplica(int groupID) {
		return (short) (EXECUTOR_ID_START + groupID * REFITConfig.EXEC_GROUP_SIZE);
	}

	// exclusive
	public static short getLastExecutionReplica(int groupID) {
		return (short) (EXECUTOR_ID_START + (groupID + 1) * REFITConfig.EXEC_GROUP_SIZE);
	}

	public static short getExecutionGroupId(short id) {
		return (short) ((id - EXECUTOR_ID_START) / REFITConfig.EXEC_GROUP_SIZE);
	}

	public static short getClientEGID(int id) {
		if (REFITConfig.CLIENT_FORCE_GROUP >= 0) {
			return REFITConfig.CLIENT_FORCE_GROUP;
		}
		for (short i = REFITConfig.ORDER_GROUP_SIZE; i < REFITConfig.TOTAL_NR_OF_REPLICAS; i++) {
			if (REFITConfig.LOCATION[id] == REFITConfig.LOCATION[i]) {
				return getExecutionGroupId(i);
			}
		}
		throw new IllegalArgumentException("No execution group for client");
	}

	public static int getClientGroup(short clientID) {
		clientID -= REFITConfig.TOTAL_NR_OF_REPLICAS;
		int group = 0;
		while (clientID >= REFITConfig.NR_OF_CLIENTS[group]) {
			clientID -= REFITConfig.NR_OF_CLIENTS[group];
			group++;
		}
		return group;
	}

	public static int countGroupSize(boolean[] group) {
		int i = 0;
		for (boolean b : group) {
			if (b) i++;
		}
		return i;
	}

	public static boolean checkInGroup(boolean[] group, short replicaID) {
		if (replicaID < 0 || replicaID >= group.length) {
			return false;
		}
		return group[replicaID];
	}

	public static short nextInGroup(boolean[] group, short replicaID) {
		if (replicaID < 0 || replicaID >= group.length) {
			throw new IllegalArgumentException("replicaID out of bounds!");
		}

		short next = replicaID;
		do {
			next = (short) ((next + 1) % group.length);
			if (group[next]) return next;
		} while (next != replicaID);

		return replicaID;
	}
}
