package refit.replica.checkpoint;

import java.nio.ByteBuffer;
import java.util.Arrays;

import refit.message.REFITMessage;
import refit.message.REFITMessageType;
import refit.message.REFITUniqueID;
import refit.util.REFITPayload;


public class REFITCheckpoint extends REFITMessage {

	static final short LIGHTWEIGHT_CHECKPOINT_MAGIC = -48;
	static final short REGULAR_CHECKPOINT_MAGIC = -1;

	public final byte[] contentHash;
	private transient REFITCheckpointState state;

	public REFITCheckpoint(long agreementSeqNr, short from, byte[] contentHash, boolean isLightweight) {
		super(REFITMessageType.CHECKPOINT, new REFITUniqueID(isLightweight ? LIGHTWEIGHT_CHECKPOINT_MAGIC : REGULAR_CHECKPOINT_MAGIC, agreementSeqNr), from);
		this.contentHash = contentHash;
		if (contentHash.length != REFITPayload.HASH_SIZE) {
			throw new AssertionError("content hash length differs from payload length");
		}
	}

	public REFITCheckpoint(ByteBuffer buffer) {
		super(REFITMessageType.CHECKPOINT, buffer);
		contentHash = new byte[REFITPayload.HASH_SIZE];
		buffer.get(contentHash);
	}

	@Override
	public AuthenticationType getAuthenticationType() {
		return AuthenticationType.SIGNATURE;
	}

	@Override
	public String toString() {
		return String.format("{%s|%d|%d}", type, from, uid.seqNr);
	}

	@Override
	protected void serialize(ByteBuffer buffer) {
		super.serialize(buffer);
		buffer.put(contentHash, 0, contentHash.length);
	}

	@Override
	protected int calculatePayloadSize() {
		return super.calculatePayloadSize() + REFITPayload.HASH_SIZE;
	}

	public boolean isLightweight() {
		return uid.nodeID == LIGHTWEIGHT_CHECKPOINT_MAGIC;
	}

	public boolean hasState() {
		return (state != null);
	}

	public REFITCheckpointState getState() {
		return state;
	}

	public boolean setState(REFITCheckpointState state) {
		if (!uid.equals(state.uid)) return false;
		if (!Arrays.equals(state.getContentHash(), contentHash)) return false;
		this.state = state;
		return true;
	}
}
