package refit.replica.execclient;

import java.util.HashMap;
import java.util.Map;

import refit.config.REFITConfig;
import refit.message.REFITReply;
import refit.message.REFITRequest;
import refit.message.REFITUniqueID;


public class REFITReplyCache {

	private final Map<Short, REFITClientRequestReplyCacheEntry> cache;

	public REFITReplyCache() {
		this.cache = new HashMap<>(REFITConfig.TOTAL_NR_OF_CLIENTS);
	}

	@Override
	public String toString() {
		return "RPYCH";
	}

	public boolean isOldUID(REFITUniqueID uid, boolean isTotalOrder) {
		short nid = (short) (2 * uid.nodeID + (isTotalOrder ? 0 : 1));
		REFITClientRequestReplyCacheEntry entry = cache.get(nid);
		if (entry == null) return false;
		return (uid.seqNr < entry.uid.seqNr);
	}

	public boolean isCurrentUID(REFITUniqueID uid, boolean isTotalOrder) {
		short nid = (short) (2 * uid.nodeID + (isTotalOrder ? 0 : 1));
		REFITClientRequestReplyCacheEntry entry = cache.get(nid);
		if (entry == null) return false;

		// only consider a request as current if we already have the request
		return (uid.seqNr == entry.uid.seqNr) && entry.request != null;
	}

	public boolean registerRequest(REFITRequest request) {
		short nid = (short) (2 * request.uid.nodeID + (request.mode.isTotalOrder() ? 0 : 1));
		// Get entry for client
		REFITClientRequestReplyCacheEntry entry = cache.get(nid);
		if (entry == null) {
			// Create new entry
			entry = new REFITClientRequestReplyCacheEntry();
			cache.put(nid, entry);
		} else {
			// Make sure that only new UIDs are stored
			if (request.uid.seqNr < entry.uid.seqNr) return false;
			// Send a client request once via the request IRMC even if we already have a reply for it
			if (request.uid.seqNr == entry.uid.seqNr && entry.request != null) return false;
		}

		// clear reply only if this is a new request
		entry.request = request;
		if (entry.uid != null && request.uid.seqNr > entry.uid.seqNr)
			entry.reply = null;

		// Update entry
		entry.uid = request.uid;
		return true;
	}

	public void storeReply(REFITReply reply) {
		short nid = (short) (2 * reply.uid.nodeID + (reply.mode.isTotalOrder() ? 0 : 1));
		// Get entry for client
		REFITClientRequestReplyCacheEntry entry = cache.get(nid);
		if (entry == null) {
			// Create new entry
			entry = new REFITClientRequestReplyCacheEntry();
			cache.put(nid, entry);
		} else {
			// Make sure that only new replies are stored
			if (reply.uid.seqNr < entry.uid.seqNr) return;
		}

		// clear request only if this is a newer reply
		if (entry.uid != null && reply.uid.seqNr > entry.uid.seqNr)
			entry.request = null;
		entry.reply = reply;

		// Store reply
		entry.uid = reply.uid;
	}

	public REFITReply getReply(REFITUniqueID uid, boolean isTotalOrder) {
		short nid = (short) (2 * uid.nodeID + (isTotalOrder ? 0 : 1));
		REFITClientRequestReplyCacheEntry entry = cache.get(nid);
		if (entry == null) return null;
		if (uid.seqNr != entry.uid.seqNr) return null;
		return entry.reply;
	}

	// ###############
	// # CACHE ENTRY #
	// ###############

	private static class REFITClientRequestReplyCacheEntry {

		public REFITUniqueID uid;
		public REFITRequest request;
		public REFITReply reply;

		@Override
		public String toString() {
			return uid.toString() + ((request != null) ? "+req" : "") + ((reply != null) ? "+repl" : "");
		}
	}

}
