package refit.replica.execution;

import refit.config.REFITConfig;
import refit.config.REFITLogger;
import refit.message.*;
import refit.message.REFITInstruction.REFITExecuteRequest;
import refit.replica.REFITBaseReplica;
import refit.replica.order.REFITOrderGroups;

public class REFITExecutionStubStage extends REFITExecutionStageBase {

	public REFITExecutionStubStage(REFITBaseReplica replica) {
		super(replica);
	}

	@Override
	public String toString() {
		return "NEXCT";
	}

	@Override
	protected void handleMessage(REFITMessage message) {
		switch (message.type) {
		case INSTRUCTION:
			handleInstruction((REFITInstruction) message);
			break;
		case UPDATE:
		case CHECKPOINT:
		case REQUEST:
			break;
		default:
			REFITLogger.logError(this, "drop message of unexpected type " + message.type);
		}
	}

	private void handleInstruction(REFITInstruction instruction) {
		// Just handle every request as the ordering may complete them out of order.
		// If the couplingStage is used, it will guarantee an ordered delivery of requests
		// this is necessary to guarantee that a leader cannot leave gaps in the local sequence numbers.

		// Store operation
		switch (instruction.instructionType) {
		case EXECUTE_REQUEST:
			REFITExecuteRequest operation = (REFITExecuteRequest) instruction;
			REFITOrderRequest request = (REFITOrderRequest) operation.message;
			int groupID = REFITOrderGroups.getGroupID(operation.agreementSeqNr);
			switch (request.type) {
			case REQUEST:
				processSingleRequest((REFITRequest) request, groupID);
				break;
			case BATCH:
				for (REFITRequest req : ((REFITBatch) request).requests)
					processSingleRequest(req, groupID);
				break;
			case REQUEST_NO_OP:
			default:
				break;
			}
			break;
		case CONFIGURATION_NOTIFICATION:
		case PROGRESS_NOTIFICATION:
			break;
		default:
			REFITLogger.logError(this, "drop instruction of unexpected type " + instruction.instructionType);
		}
	}

	private void processSingleRequest(REFITRequest request, int groupID) {
		// reply stub
		REFITReply reply = new REFITReply(request.uid, replica.id, (short) groupID, request.mode);
		if (REFITConfig.COLLECT_MESSAGE_STATISTICS) reply.getContext().statistics = request.getContext().statistics;
		// forward reply, the client stage should be able to cope with repeated no ops
		replySenders[groupID].broadcast(reply);
	}
}
