package refit.replica.execution;

import java.nio.ByteBuffer;

import refit.config.REFITConfig;
import refit.replica.checkpoint.REFITCheckpointObject;
import refit.util.REFITPayload;


public class REFITExecutorState extends REFITCheckpointObject {

	public final long[] nodeProgresses;
	public final byte[][] replyHashes;
	public final long[] nodeSeqs;
	public final int targetMinView;
	public final long targetAfterSeq;
	public final long targetProbeCounter;


	public REFITExecutorState(long[] nodeProgresses, byte[][] replyHashes, long[] nodeSeqs, int targetMinView, long targetAfterSeq, long targetProbeCounter) {
		this.targetMinView = targetMinView;
		this.targetAfterSeq = targetAfterSeq;
		this.targetProbeCounter = targetProbeCounter;

		this.nodeProgresses = new long[REFITConfig.ADDRESSES.length];
		System.arraycopy(nodeProgresses, 0, this.nodeProgresses, 0, this.nodeProgresses.length);

		if (replyHashes.length != REFITConfig.ADDRESSES.length)
			throw new AssertionError("Reply count differs from client count");
		for (byte[] replyHash : replyHashes) {
			if (replyHash.length != REFITPayload.HASH_SIZE)
				throw new AssertionError("Reply hash has unexpected length");
		}
		this.replyHashes = new byte[replyHashes.length][];
		System.arraycopy(replyHashes, 0, this.replyHashes, 0, replyHashes.length);

		this.nodeSeqs = new long[REFITConfig.ADDRESSES.length];
		System.arraycopy(nodeSeqs, 0, this.nodeSeqs, 0, this.nodeSeqs.length);

	}

	public REFITExecutorState(ByteBuffer buffer) {
		super(buffer);

		this.targetMinView = buffer.getInt();
		this.targetAfterSeq = buffer.getLong();
		this.targetProbeCounter = buffer.getLong();

		this.replyHashes = new byte[REFITConfig.ADDRESSES.length][];
		for (int i = 0; i < replyHashes.length; i++) {
			byte[] replyHash = new byte[REFITPayload.HASH_SIZE];
			buffer.get(replyHash);
			this.replyHashes[i] = replyHash;
		}

		this.nodeProgresses = new long[REFITConfig.ADDRESSES.length];
		for (int i = 0; i < nodeProgresses.length; i++) nodeProgresses[i] = buffer.getLong();

		this.nodeSeqs = new long[REFITConfig.ADDRESSES.length];
		for (int i = 0; i < nodeSeqs.length; i++) nodeSeqs[i] = buffer.getLong();

		messageSize = buffer.position();
		if (buffer.position() != buffer.capacity()) {
			throw new AssertionError("invalid executor state");
		}
	}

	@Override
	public String toString() {
		return "{REFITExecutorState}";
	}

	@Override
	protected void serialize(ByteBuffer buffer) {
		buffer.putInt(targetMinView);
		buffer.putLong(targetAfterSeq);
		buffer.putLong(targetProbeCounter);

		for (byte[] replyHash : replyHashes) {
			buffer.put(replyHash);
		}
		for (long clientProgress : nodeProgresses) buffer.putLong(clientProgress);
		for (long clientSeq : nodeSeqs) buffer.putLong(clientSeq);
	}

	@Override
	protected int calculatePayloadSize() {
		return Integer.BYTES + Long.BYTES + Long.BYTES
				+ REFITConfig.ADDRESSES.length * REFITPayload.HASH_SIZE
				+ (Long.BYTES + Long.BYTES) * nodeProgresses.length;
	}
}
