package refit.replica.mcclient;

import refit.config.REFITConfig;
import refit.config.REFITLogger;
import refit.message.REFITInstruction;
import refit.message.REFITInstruction.REFITChannelProgress;
import refit.message.REFITInstruction.REFITChannelSkip;
import refit.message.REFITMessage;
import refit.message.REFITRequest;
import refit.replica.REFITBaseReplica;
import refit.replica.REFITObserver;
import refit.replica.REFITReplicaGroups;
import refit.replica.REFITStage;
import refit.replica.multichannel.REFITChannelItem;
import refit.scheduler.REFITSchedulerTaskType;

public class REFITMCMultiRemoteClientStage extends REFITStage {

	private static final int CLIENT_TOKENS = 1;

	public final REFITObserver<REFITRequest> requestSender;
	public final REFITObserver<REFITChannelSkip> skipSender;
	public final REFITObserver<REFITChannelProgress> progressSender;
	public final REFITObserver<REFITMessage>[] channelMsgSender;

	// agreement side, dispatch channel messages to IRMC and represent the client
	public REFITMCMultiRemoteClientStage(REFITBaseReplica replica) {
		super(REFITSchedulerTaskType.EXEC_COUPLING_STAGE, replica);
		requestSender = new REFITObserver<>();
		skipSender = new REFITObserver<>();
		progressSender = new REFITObserver<>();
		//noinspection unchecked
		channelMsgSender = (REFITObserver<REFITMessage>[]) new REFITObserver[REFITReplicaGroups.EXEC_COUNT];
		for (int i = 0; i < channelMsgSender.length; i++) {
			channelMsgSender[i] = new REFITObserver<>();
		}
	}

	@Override
	public String toString() {
		return "MCLI-DST";
	}

	@Override
	public void init() {
		super.init();
		for (int i = 0; i < REFITConfig.TOTAL_NR_OF_CLIENTS; i++) {
			progressSender.broadcast(new REFITChannelProgress((short) i, 0, -1, CLIENT_TOKENS));
		}
	}

	@Override
	protected void handleMessage(REFITMessage message) {
		switch (message.groupType()) {
		case CHANNEL_ITEM:
			REFITChannelItem item = (REFITChannelItem) message;
			requestSender.broadcast((REFITRequest) item.content);
			break;
		case CHANNEL_MSG:
			handleChannelMsg(message);
			break;
		case INSTRUCTION:
			handleInstruction((REFITInstruction) message);
			break;
		default:
			REFITLogger.logError(this, "drop message of unexpected type " + message.type);
		}
	}

	private void handleChannelMsg(REFITMessage message) {
		int groupId = REFITReplicaGroups.getExecutionGroupId(message.from);
		if (groupId < 0 || groupId >= channelMsgSender.length) {
			REFITLogger.logWarning(this, "Dropping message from invalid sender " + message);
			return;
		}

		channelMsgSender[groupId].broadcast(message);
	}

	private void handleInstruction(REFITInstruction message) {
		switch (message.instructionType) {
		case CHANNEL_PROGRESS:
			progressSender.broadcast((REFITChannelProgress) message);
			break;
		case CHANNEL_SKIP:
			skipSender.broadcast(((REFITChannelSkip) message));
			break;
		default:
			REFITLogger.logError(this, "drop instruction of unexpected type " + message.type);
		}
	}
}
