package refit.replica.mccoupling;

import refit.config.REFITConfig;
import refit.config.REFITLogger;
import refit.message.REFITInstruction;
import refit.message.REFITInstruction.*;
import refit.message.REFITMessage;
import refit.replica.REFITBaseReplica;
import refit.replica.REFITObserver;
import refit.replica.REFITReplicaGroups;
import refit.replica.REFITStage;
import refit.replica.multichannel.REFITChannelItem;
import refit.replica.order.REFITStubOrderProtocol;
import refit.scheduler.REFITSchedulerTaskType;

public class REFITMCExecutorCouplingStage extends REFITStage {

	public final REFITObserver<REFITExecuteRequest> requestSender;
	public final REFITObserver<REFITChannelProgress> progressSender;
	public final REFITObserver<REFITConfigurationNotification> configSender;
	public final REFITObserver<REFITCheckpointAssist> assistSender;

	public REFITMCExecutorCouplingStage(REFITBaseReplica replica) {
		super(REFITSchedulerTaskType.EXEC_COUPLING_STAGE, replica);
		requestSender = new REFITObserver<>();
		progressSender = new REFITObserver<>();
		configSender = new REFITObserver<>();
		assistSender = new REFITObserver<>();
	}

	@Override
	public String toString() {
		return "MCPL-DST";
	}

	@Override
	public void init() {
		super.init();
		progressSender.broadcast(new REFITChannelProgress((short) 0, 0, -1, REFITConfig.MAXIMUM_EXECUTION_INSTANCES_IN_PROGRESS));

		// send configuration notification with stub protocol to execution
		configSender.broadcast(new REFITConfigurationNotification(new REFITStubOrderProtocol(), -1, -1));
	}

	@Override
	protected void handleMessage(REFITMessage message) {
		switch (message.type) {
		case CHANNEL_ITEM:
			handleResult(((REFITChannelItem) message));
			break;
		case INSTRUCTION:
			handleInstruction(((REFITInstruction) message));
			break;
		default:
			REFITLogger.logError(this, "drop message of unexpected type " + message.type);
		}
	}

	private void handleResult(REFITChannelItem message) {
		requestSender.broadcast(new REFITExecuteRequest(message.uid.seqNr, 0, message.content, null));

		int groupId = REFITReplicaGroups.getExecutionGroupId(replica.id);
		if (groupId == 0 && message.uid.seqNr == REFITConfig.GROUP_LATE_START_SEQNR) {
			assistSender.broadcast(new REFITCheckpointAssist(REFITConfig.GROUP_LATE_START_GID,
					REFITConfig.GROUP_LATE_START_SEQNR, REFITReplicaGroups.getFirstExecutionReplica(groupId)));
		}
	}

	private void handleInstruction(REFITInstruction message) {
		switch (message.instructionType) {
		case PROGRESS_NOTIFICATION:
			// Forward progress from checkpoint stage
			REFITProgressNotification progress = (REFITProgressNotification) message;
			progressSender.broadcast(new REFITChannelProgress((short) 0, progress.agreementSeqNr, -1, REFITConfig.MAXIMUM_EXECUTION_INSTANCES_IN_PROGRESS));
			break;
		case CHANNEL_SKIP:
			throw new InternalError("Unexpected instruction " + message + " handling not implemented");
		default:
			REFITLogger.logError(this, "drop instruction of unexpected type " + message.instructionType);
		}
	}

}
