package refit.replica.mcmicro;

import refit.config.REFITConfig;
import refit.config.REFITLogger;
import refit.message.REFITInstruction.REFITChannelProgress;
import refit.message.REFITMessage;
import refit.replica.REFITBaseReplica;
import refit.replica.REFITObserver;
import refit.replica.REFITStage;
import refit.replica.multichannel.REFITChannelItem;
import refit.scheduler.REFITSchedulerTaskType;

public class REFITChannelMicroReceiverStage extends REFITStage {

	public final REFITObserver<REFITChannelProgress> progressSender;

	private long lastAck = -1;

	public REFITChannelMicroReceiverStage(REFITBaseReplica replica) {
		super(REFITSchedulerTaskType.EXEC_COUPLING_STAGE, replica);
		progressSender = new REFITObserver<>();
	}

	@Override
	public String toString() {
		return "CMIC-DST";
	}

	@Override
	public void init() {
		super.init();
		progressSender.broadcast(new REFITChannelProgress((short) 0, 0, -1, REFITConfig.MAXIMUM_EXECUTION_INSTANCES_IN_PROGRESS));
	}

	@Override
	protected void handleMessage(REFITMessage message) {
		switch (message.type) {
		case CHANNEL_ITEM:
			handleResult(((REFITChannelItem) message));
			break;
		default:
			REFITLogger.logError(this, "drop message of unexpected type " + message.type);
		}
	}

	private void handleResult(REFITChannelItem message) {
//		REFITLogger.logWarning(this, "" + message.content.uid.seqNr);
		long ackSeq = message.content.uid.seqNr / REFITConfig.CHECKPOINT_INTERVAL * REFITConfig.CHECKPOINT_INTERVAL;
		if (ackSeq > lastAck) {
			lastAck = ackSeq;
			progressSender.broadcast(new REFITChannelProgress((short) 0, lastAck, -1, REFITConfig.MAXIMUM_EXECUTION_INSTANCES_IN_PROGRESS));
			REFITLogger.logWarning(this, "" + message.content.uid.seqNr);
		}
	}
}
