package refit.replica.mcmicro;

import refit.config.REFITConfig;
import refit.config.REFITLogger;
import refit.message.REFITInstruction;
import refit.message.REFITInstruction.REFITChannelProgress;
import refit.message.REFITMessage;
import refit.message.REFITMicro;
import refit.message.REFITUniqueID;
import refit.replica.REFITBaseReplica;
import refit.replica.REFITObserver;
import refit.replica.REFITStage;
import refit.replica.multichannel.REFITChannelItem;
import refit.scheduler.REFITSchedulerTaskType;

public class REFITChannelMicroSenderStage extends REFITStage {

	public final REFITObserver<REFITChannelItem> channelRequestSender;

	private long nextSeqNr = -1;
	private long seqNrLimit = 0;
	private boolean init = false;

	public REFITChannelMicroSenderStage(REFITBaseReplica replica) {
		super(REFITSchedulerTaskType.COUPLING_STAGE, replica);
		channelRequestSender = new REFITObserver<>();
	}

	@Override
	public String toString() {
		return "CMIC-SRC";
	}

	@Override
	protected void handleMessage(REFITMessage message) {
		switch (message.type) {
		case INSTRUCTION:
			handleInstruction((REFITInstruction) message);
			break;
		default:
			REFITLogger.logError(this, "drop message of unexpected type " + message.type);
		}
	}

	private void handleInstruction(REFITInstruction message) {
		switch (message.instructionType) {
		case CHANNEL_PROGRESS:
			REFITChannelProgress progress = (REFITChannelProgress) message;
			nextSeqNr = Math.max(nextSeqNr, progress.baseSeqNr);
			seqNrLimit = progress.baseSeqNr + progress.tokens;
			break;
		default:
			REFITLogger.logError(this, "drop instruction of unexpected type " + message.type);
		}
	}

	@Override
	protected void stageComplete() {
		if (!init) {
			if (!hasTimeout()) {
				setTimeout(2000);
			}
			return;
		}

		// linear iteration over sequence numbers is only necessary for proper timeout handling
		while (nextSeqNr < seqNrLimit) {
			REFITMicro msg = new REFITMicro(new REFITUniqueID((short) 42, nextSeqNr), (short) 42, (short) REFITConfig.REQUEST_SIZE);
			REFITChannelItem channelRequest = new REFITChannelItem(nextSeqNr, (short) 0, msg);
			channelRequestSender.broadcast(channelRequest);
			nextSeqNr++;
		}
	}

	@Override
	public void timeout() {
		init = true;
	}
}
