package refit.replica.multichannel;

import java.util.Arrays;

import refit.config.REFITConfig;

public class REFITChannelAckHandler {
	private final int requiredAcks;
	private final long[] ackedSeqNrs;
	private long mergedSeqNr;

	enum AckStatus {
		INVALID,
		OK,
		CHANGED
	}

	public REFITChannelAckHandler(int requiredAcks) {
		this.requiredAcks = requiredAcks;
		ackedSeqNrs = new long[REFITConfig.TOTAL_NR_OF_REPLICAS];
		Arrays.fill(ackedSeqNrs, -2);
		mergedSeqNr = -1;
	}

	public AckStatus handleProgress(short from, long seqNr) {
		if (seqNr <= ackedSeqNrs[from]) {
			return AckStatus.INVALID;
		}
		ackedSeqNrs[from] = seqNr;

		long[] progress = ackedSeqNrs.clone();
		Arrays.sort(progress);
		long oldProgress = mergedSeqNr;
		mergedSeqNr = progress[progress.length - requiredAcks];

		boolean progressChanged = oldProgress != mergedSeqNr;

		return progressChanged ? AckStatus.CHANGED : AckStatus.OK;
	}

	public long getMergedSeqNr() {
		return mergedSeqNr;
	}

	public long[] getProgress() {
		return ackedSeqNrs.clone();
	}
}
