package refit.replica.multichannel;

import java.nio.ByteBuffer;
import java.util.List;

import refit.config.REFITConfig;
import refit.config.REFITLogger;
import refit.message.REFITMessageAuthentication;
import refit.message.REFITUniqueID;
import refit.util.REFITBallotBox;

public class REFITFullShareCertificate implements REFITChannelInboxCertificate<REFITShare> {

	private final REFITMessageAuthentication mac;

	private final REFITUniqueID uid;
	private boolean[] legitimateSenders;
	private final REFITBallotBox<Short, ByteBuffer, REFITShare> shares;

	private REFITShare[] result;

	public REFITFullShareCertificate(REFITUniqueID uid, REFITMessageAuthentication mac, boolean[] legitimateSenders, int senderFaults) {
		this.mac = mac;

		this.uid = uid;
		this.legitimateSenders = legitimateSenders;
		this.shares = new REFITBallotBox<>(senderFaults + 1);
	}

	@Override
	public boolean add(REFITShare share) {
		// Check reply
		if (share == null)
			return false;
		if (!uid.equals(share.uid))
			return false;
		if (share.isHashOnly())
			return false;
		if (!mac.verifySignature(share, legitimateSenders))
			return false;

		// Prepare vote
		ByteBuffer vote = ByteBuffer.wrap(share.getContent().getHash());

		// Cast vote
		boolean success = shares.add(share.from, vote, share);
		if (success) checkResult();
		return success;
	}

	@Override
	public boolean isStable() {
		return (result != null);
	}

	@Override
	public REFITShare[] getResult() {
		return result;
	}

	private void checkResult() {
		if (isStable()) return;
		if (REFITConfig.ENABLE_DEBUG_CHECKS) if (shares.votesDiffer())
			REFITLogger.logWarning("ACS-DST", "received different executes for request " + uid);
		List<REFITShare> resultSet = shares.getDecidingBallots();
		if (resultSet == null) return;
		result = resultSet.toArray(new REFITShare[0]);
	}
}
