package refit.replica.order;

import java.nio.ByteBuffer;

import refit.message.REFITMessage;
import refit.message.REFITMessageType;
import refit.message.REFITUniqueID;


public abstract class REFITOrderProtocolMessage extends REFITMessage {

	private static final int ORDER_HEADER_SIZE = Integer.BYTES;

	public final int viewID;


	protected REFITOrderProtocolMessage(REFITMessageType type, long instanceID, int viewID, short from) {
		super(type, new REFITUniqueID(from, instanceID), from);
		this.viewID = viewID;
	}

	protected REFITOrderProtocolMessage(REFITMessageType expectedType, ByteBuffer buffer) {
		super(expectedType, buffer);
		this.viewID = buffer.getInt();
	}

	@Override
	public AuthenticationType getAuthenticationType() {
		return AuthenticationType.NOT_SERIALIZABLE;
	}

	@Override
	public REFITMessageType groupType() {
		// Allow selection of all subtypes by the message type ORDER
		return REFITMessageType.ORDER;
	}

	@Override
	public String toString() {
		return String.format("{%s|%d|%s|%d}", type, from, uid, viewID);
	}

	@Override
	protected void serialize(ByteBuffer buffer) {
		super.serialize(buffer);
		buffer.putInt(viewID);
	}

	@Override
	protected int getHeaderSize() {
		return (super.getHeaderSize() + ORDER_HEADER_SIZE);
	}

	@Override
	protected int calculatePayloadSize() {
		return (super.calculatePayloadSize() + ORDER_HEADER_SIZE);
	}


}
