package refit.replica.order;

import refit.message.REFITMessage;
import refit.message.REFITMessageType;
import refit.replica.REFITBaseReplica;

public abstract class REFITOrderProtocolTransitionInstance {
	protected final REFITBaseReplica replica;
	protected final REFITOrderStageSlot slot;

	public long instanceID;
	public int viewID;

	public REFITOrderProtocolTransitionInstance(REFITOrderStageSlot slot) {
		this.replica = slot.replica;
		this.slot = slot;
		this.instanceID = -1L;
		this.viewID = -1;
	}

	// ##############
	// # LIFE CYCLE #
	// ##############

	public void init(long instanceID, int viewID) {
		this.instanceID = instanceID;
		this.viewID = viewID;
	}

	public abstract void updateProtocolData(REFITOrderProtocolData data);

	public abstract void updateView(int viewID);

	// execution may be triggered by unexpected messages from other replicas!
	public abstract void execute();

	protected void synchronizeNewView(int newViewID) {
		slot.synchronizeNewView(newViewID);
	}

	protected void learnCompletableViewChange() {
		slot.learnCompletableViewChange();
	}

	protected void learnViewChange(REFITOrderProtocol protocol, REFITOrderProtocolInstance successorInstance) {
		slot.learnViewChange(protocol, successorInstance);
	}


	// ####################
	// # MESSAGE HANDLING #
	// ####################

	protected REFITMessage fetchMessage(REFITMessageType type) {
		return slot.fetchMessage(type, -1);
	}
}
