package refit.replica.update;

import java.util.Arrays;
import java.util.List;

import refit.config.REFITConfig;
import refit.message.REFITMessageAuthentication;
import refit.message.REFITUniqueID;
import refit.util.REFITBallotBox;


public class REFITUpdateCertificate {

	public final REFITUniqueID uid;
	private boolean[] legitimateSenders;
	public final REFITBallotBox<Short, REFITUpdateCertificateVote, REFITUpdateMessage> updates;
	public REFITUpdateMessage result;


	public REFITUpdateCertificate(REFITUniqueID uid, boolean[] legitimateSenders) {
		this.uid = uid;
		this.legitimateSenders = legitimateSenders;
		this.updates = new REFITBallotBox<>(REFITConfig.UPDATE_STABILITY_THRESHOLD);
		this.result = null;
	}


	public boolean add(REFITUpdateMessage update, REFITMessageAuthentication mac) {
		// Check update
		if (update == null) return false;
		if (!uid.equals(update.uid)) return false;
		if (updates.hasVoted(update.from)) return false;
		if (!mac.verifyMulticastMAC(update, legitimateSenders)) return false;

		// Prepare vote
		REFITUpdateCertificateVote vote = new REFITUpdateCertificateVote(update.agreementSeqNr(), update.getPayloadHash());

		// Cast vote
		boolean success = updates.add(update.from, vote, update);
		if (success) checkResult();
		return success;
	}

	public boolean isStable() {
		return (result != null);
	}

	private void checkResult() {
		if (isStable()) return;
		List<REFITUpdateMessage> correctUpdates = updates.getDecidingBallots();
		if (correctUpdates == null) return;
		for (REFITUpdateMessage update : correctUpdates) {
			if (!update.isFullUpdate()) continue;
			result = update;
			break;
		}
	}


	// ###############
	// # UPDATE VOTE #
	// ###############

	private static class REFITUpdateCertificateVote {

		private final long agreementSeqNr;
		private final byte[] payloadHash;


		public REFITUpdateCertificateVote(long agreementSeqNr, byte[] payloadHash) {
			this.agreementSeqNr = agreementSeqNr;
			this.payloadHash = payloadHash;
		}


		@Override
		public boolean equals(Object object) {
			if (this == object) return true;
			if (object == null || getClass() != object.getClass()) return false;
			REFITUpdateCertificateVote other = (REFITUpdateCertificateVote) object;
			return agreementSeqNr == other.agreementSeqNr &&
					Arrays.equals(payloadHash, other.payloadHash);
		}

		@Override
		public int hashCode() {
			return (int) agreementSeqNr;
		}

	}

}
