package refit.test;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class HashBenchmark {

	public static void main(String[] args) {
		final String digestName = "SHA-256";
		final int DATA_SIZE = 1024;
		final int ITERATIONS = 50000;

		final int threads = 8;
		ExecutorService executor = Executors.newFixedThreadPool(threads);

		for (int i = 0; i < threads; i++) {
			executor.execute(new Runnable() {
				@Override
				public void run() {
					MessageDigest digest;
					try {
						digest = MessageDigest.getInstance(digestName);
					} catch (NoSuchAlgorithmException e) {
						e.printStackTrace();
						return;
					}

					for (int round = 0; round < 100; round++) {
						byte[] data = new byte[DATA_SIZE];
						long timeBefore = System.nanoTime();
						for (int j = 0; j < ITERATIONS; j++) {
							digest.update(data);
							digest.digest();
						}
						long timeAfter = System.nanoTime();
						long timeDiff = timeAfter - timeBefore;
						System.out.printf("%s with %d bytes took: %d ns\n", digestName,
								DATA_SIZE, timeDiff / ITERATIONS);
					}
				}
			});
		}


		executor.shutdown();
		try {
			executor.awaitTermination(5, TimeUnit.MINUTES);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}
