package refit.test;

import java.io.IOException;
import java.nio.channels.Selector;
import java.util.concurrent.atomic.AtomicBoolean;

public class SelectorMicrobenchmark {
	public static void main(String[] args) {
		Selector selector = null;
		try {
			selector = Selector.open();
		} catch (IOException ioe) {
			ioe.printStackTrace();
			System.exit(100);
		}

		final Selector sel = selector;
		final AtomicBoolean isWoken = new AtomicBoolean(false);
		final AtomicBoolean isFinished = new AtomicBoolean(false);
		final int wakeCount = 100000000;

		Thread producer = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int counter = 0; counter < wakeCount; counter++) {
					// INSERT: queue data
					if (isWoken.compareAndSet(false, true)) {
						sel.wakeup();
					}
				}
				isFinished.set(true);
				sel.wakeup();
			}
		});

		Thread consumer = new Thread(new Runnable() {
			@Override
			public void run() {
				int counter = 0;
				while (!isFinished.get()) {
					try {
						sel.select();
					} catch (IOException e) {
						e.printStackTrace();
					}
					if (isWoken.compareAndSet(true, false)) {
						// INSERT: process and dequeue data
						counter++;
					}
				}
				System.out.println("Select iterations: " + counter);
			}
		});

		long startTime = System.nanoTime();
		producer.start();
		consumer.start();

		try {
			producer.join();
			consumer.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		long endTime = System.nanoTime();
		long diff = endTime - startTime;

		System.out.println("Finished in " + diff / 1000 + " us");
		long throughput = (long) ((double) wakeCount / diff * 1000000000);
		System.out.println("Throughput: " + throughput + " wakeups calls per second");
	}
}
