package refit.replica.fuzzycheckpoint;

import java.util.Map;
import java.util.Set;

import refit.message.REFITMessage;
import refit.message.REFITMessageType;
import refit.replica.checkpoint.REFITCheckpointObject;
import refit.replica.execution.REFITExecutorState;

public abstract class REFITFuzzyCheckpointNotification extends REFITMessage {

	// #####################
	// # INSTRUCTION TYPES #
	// #####################

	public enum REFITFuzzyCheckpointNotificationType {
		// trigger start of fuzzy collection EXEC -> FCP
		START,
		// last request of current interval processed EXEC -> FCP
		FINISH,
		// concurrent state collection done FCP -> EXEC
		COLLECTION_DONE
	}

	// ########################
	// # ABSTRACT INSTRUCTION #
	// ########################

	public REFITFuzzyCheckpointNotificationType notificationType;

	private REFITFuzzyCheckpointNotification(REFITFuzzyCheckpointNotificationType notificationType) {
		super(REFITMessageType.FUZZY_CHECKPOINT_NOTIFICATION, null, (short) -1);
		this.notificationType = notificationType;
	}

	@Override
	public AuthenticationType getAuthenticationType() {
		return AuthenticationType.NOT_SERIALIZABLE;
	}

	@Override
	public String toString() {
		return String.format("{%-16s}", notificationType);
	}


	// #########################
	// # SPECIFIC INSTRUCTIONS #
	// #########################

	public static class REFITFuzzyCheckpointStartNotification extends REFITFuzzyCheckpointNotification {
		// triggering sequence number
		public final long startSeqNr;
		public final Set<?> changelist;

		public REFITFuzzyCheckpointStartNotification(long startSeqNr, Set<?> changelist) {
			super(REFITFuzzyCheckpointNotificationType.START);
			this.startSeqNr = startSeqNr;
			this.changelist = changelist;
		}
	}

	public static class REFITFuzzyCheckpointFinishNotification extends REFITFuzzyCheckpointNotification {
		// last sequence number of the finished checkpoint
		public final long lastSeqNr;
		public final REFITExecutorState metaData;
		public final Map<Object, ? extends REFITCheckpointObject> changedObjects;
		public final REFITCheckpointObject sysAppCheckpoint;

		public REFITFuzzyCheckpointFinishNotification(long lastSeqNr, REFITExecutorState metaData,
		                                              REFITCheckpointObject sysAppCheckpoint, Map<Object, ? extends REFITCheckpointObject> changedObjects) {
			super(REFITFuzzyCheckpointNotificationType.FINISH);
			this.lastSeqNr = lastSeqNr;
			this.metaData = metaData;
			this.changedObjects = changedObjects;
			this.sysAppCheckpoint = sysAppCheckpoint;
		}
	}

	public static class REFITFuzzyCollectionDoneNotification extends REFITFuzzyCheckpointNotification {
		public REFITFuzzyCollectionDoneNotification() {
			super(REFITFuzzyCheckpointNotificationType.COLLECTION_DONE);
		}
	}
}
